@@ -15,7 +15,7 @@ amount of other code (which should eventually be placed in a sort of
 
 require_once 'slb-interface.php';
 
-define ('RE_STATIC_URI', '#^([[:alpha:]]+)/(?:[[:alpha:]]+/)*[[:alnum:]\._-]+\.([[:alpha:]]+)$#');
+define ('RE_STATIC_URI', '#^([[:alpha:]]+)/(?:[[:alnum:]]+[[:alnum:]_.-]*/)*[[:alnum:]\._-]+\.([[:alpha:]]+)$#');
 
 function dispatchImageRequest()
 {
@@ -337,7 +337,7 @@ function renderImagePreview ($file_id)
 
 function printStatic404()
 {
-	header ('404 Not Found');
+	header ('HTTP/1.0 404 Not Found');
 ?><!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
 <html><head>
 <title>404 Not Found</title>
